@@ -1,6 +1,6 @@
 #######################################################################
 # TSP - Traveling Salesperson Problem
-# Copyrigth (C) 2011 Michael Hahsler and Kurt Hornik
+# Copyright (C) 2011 Michael Hahsler and Kurt Hornik
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -107,17 +107,16 @@ NULL
 
 
 ## prepare distances as integers in the appropriate range [0..MAX]
-.prepare_dist_concorde <- function(x, MAX, precision) {
+.prepare_dist_concorde <- function(x, MAX, precision, verbose = FALSE) {
   ## handle inf
   x <- .replaceInf(x)
 
   ## fix neg. values
   min_x <- min(x)
   if (min_x < 0) {
+    if (verbose)
     warning(
-      "TSP contains negative distances. Shifting distances by subtracting the minimum.",
-      immediate. = TRUE
-    )
+      "pTSP contains negative distances (maybe the result of a reformulation from ATSP). Shifting distances by subtracting the minimum.", immediate. = FALSE)
     x <- x - min_x
   }
 
@@ -163,20 +162,6 @@ tsp_concorde <- function(x, control = NULL) {
     )
   )
 
-  ## check x
-  if (inherits(x, "TSP")) {
-    #if(n_of_cities(x) < 10) MAX <- 2^15 - 1 else MAX <- 2^31 - 1
-    ### MFH: concorde may overflow with 2^31-1
-    if (n_of_cities(x) < 10)
-      MAX <- 2 ^ 15 - 1
-    else
-      MAX <- 2 ^ 28 - 1
-    x <- .prepare_dist_concorde(x, MAX, control$precision)
-
-  } else if (inherits(x, "ETSP")) {
-    ## nothing to do!
-  } else
-    stop("Concorde only handles TSP and ETSP.")
 
 
   ## get temp files and change working directory
@@ -193,8 +178,23 @@ tsp_concorde <- function(x, control = NULL) {
   tmp_file_in  <- paste(temp_file, ".dat", sep = "")
   tmp_file_out <- paste(temp_file, ".sol", sep = "")
 
-  ## precision is already handled!
-  write_TSPLIB(x, file = tmp_file_in, precision = 0)
+  ## check x
+  if (inherits(x, "TSP")) {
+    #if(n_of_cities(x) < 10) MAX <- 2^15 - 1 else MAX <- 2^31 - 1
+    ### MFH: concorde may overflow with 2^31-1
+    if (n_of_cities(x) < 10)
+      MAX <- 2 ^ 15 - 1
+    else
+      MAX <- 2 ^ 28 - 1
+    x <- .prepare_dist_concorde(x, MAX, precision = control$precision, verbose = control$verbose)
+    ## precision is already handled!
+    write_TSPLIB(x, file = tmp_file_in, precision = 0)
+
+  } else if (inherits(x, "ETSP")) {
+    ## nothing to do!
+    write_TSPLIB(x, file = tmp_file_in, precision = control$precision)
+  } else
+    stop("Concorde only handles TSP and ETSP.")
 
   ## change working directory
 
@@ -214,7 +214,6 @@ tsp_concorde <- function(x, control = NULL) {
       FALSE,
   )
 
-
   if (!file.access(tmp_file_out) == 0)
     stop(
       "Concorde has not produced a result file.\nIs concorde properly installed? (see ? Concorde)\nDid Concorde finish without an error or being interupted?"
@@ -261,7 +260,7 @@ tsp_linkern <- function(x, control = NULL) {
   if (inherits(x, "TSP")) {
     #MAX <- 2^31 - 1
     MAX <- 2 ^ 28 - 1
-    x <- .prepare_dist_concorde(x, MAX, control$precision)
+    x <- .prepare_dist_concorde(x, MAX, precision = control$precision, verbose = control$verbose)
 
   } else if (inherits(x, "ETSP")) {
     ## nothing to do
